'use strict';

import React from "react";
import PropTypes from "prop-types";
import I18nWrapper from "../i18n/I18nWrapper";
import injectIntl from "../utils/injectIntl";

const DEFAULT_THRESHOLD = 120;

class CollapsibleText extends React.Component {

    constructor(props) {
        super(props);
        this.i18n = props.i18n;
        this.state = {
            expanded: props.defaultExpanded
        };
    }

    _onToggle = () => {
        this.setState({expanded: !this.state.expanded});
    };

    _getTextPreview() {
        const threshold = this.props.maxLength ? this.props.maxLength : DEFAULT_THRESHOLD;
        if (!this.props.text) {
            return '';
        }
        return this.props.text.length > threshold ? (this.props.text.substring(0, threshold) + '...') : this.props.text;
    }

    _getText() {
        return this.state.expanded ? this.props.text : this._getTextPreview();
    }

    _isShortened() {
        const threshold = this.props.maxLength ? this.props.maxLength : DEFAULT_THRESHOLD;
        return this.props.text && this.props.text.length > threshold;
    }

    render() {
        const style = this.state.expanded ? {whiteSpace: 'pre-wrap'} : {},
            title = this._isShortened(this.props.text) ? this.i18n('collapsible.expand.tooltip') : this.props.text;
        return <div title={title} onClick={this._onToggle} style={style}>
            {this._getText()}
        </div>;
    }
}

CollapsibleText.propTypes = {
    text: PropTypes.string,
    defaultExpanded: PropTypes.bool
};

CollapsibleText.defaultProps = {
    defaultExpanded: false
};

export default injectIntl(I18nWrapper(CollapsibleText));
