package cz.cvut.kbss.reporting.rest;

import cz.cvut.kbss.reporting.model.Severity;
import cz.cvut.kbss.reporting.rest.handler.ErrorInfo;
import cz.cvut.kbss.reporting.rest.handler.RestExceptionHandler;
import cz.cvut.kbss.reporting.rest.util.RawJson;
import cz.cvut.kbss.reporting.service.OptionsService;
import cz.cvut.kbss.reporting.util.Constants;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import static org.hamcrest.CoreMatchers.containsString;
import static org.junit.Assert.*;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

public class OptionsControllerTest extends BaseControllerTestRunner {

    @Mock
    private OptionsService serviceMock;

    @InjectMocks
    private OptionsController controller;

    @Before
    public void setUp() {
        MockitoAnnotations.initMocks(this);
        super.setUp();
        this.mockMvc = MockMvcBuilders.standaloneSetup(controller).setControllerAdvice(new RestExceptionHandler())
                                      .build();
    }

    @Test
    public void getOptionsReturnsOptionsProvidedByService() throws Exception {
        final String type = "severity";
        final RawJson options = new RawJson();
        final StringBuilder sb = new StringBuilder("[");
        for (Severity s : Severity.values()) {
            if (sb.length() > 1) {
                sb.append(',');
            }
            sb.append('\'').append(s.toString()).append('\'');
        }
        sb.append(']');
        options.setValue(sb.toString());
        when(serviceMock.getOptions(type)).thenReturn(options);
        final MvcResult mvcResult = mockMvc
                .perform(get("/options").param(Constants.OPTIONS_TYPE_QUERY_PARAM, type))
                .andExpect(status().isOk()).andReturn();
        final String result = mvcResult.getResponse().getContentAsString();
        for (Severity s : Severity.values()) {
            assertTrue(result.contains(s.toString()));
        }
    }

    @Test
    public void getOptionsThrowsBadRequestForUnknownOptionType() throws Exception {
        final String optionType = "unknownType";
        when(serviceMock.getOptions(optionType))
                .thenThrow(new IllegalArgumentException("Unsupported option type " + optionType + "."));
        final MvcResult mvcResult = mockMvc
                .perform(get("/options").param(Constants.OPTIONS_TYPE_QUERY_PARAM, optionType))
                .andExpect(status().isConflict()).andReturn();
        final ErrorInfo errorInfo = readValue(mvcResult, ErrorInfo.class);
        assertNotNull(errorInfo);
        assertThat(errorInfo.getMessage(), containsString("Unsupported option type " + optionType));
    }
}